'use strict';
var path = require('path');
import IO from 'koa-socket';
const cors = require('kcors');
const io = new IO();
global.spice = {};

require('dotenv').config();

var validate = require('koa-validation');
var bodyParser = require('koa-bodyparser');
const koaBody = require('koa-body');
spice.root_path = path.resolve('./build/');
spice.module_root_path = path.resolve(__dirname);
import Event from './events/Event';
import Model from './models/SpiceModel';
import Crypt from './utility/Crypt';
import Serializer from './utility/Serializer';
import RestHelper from './utility/RestHelper';
import Status from '../cli/lib/status';
import Mail from './mail/Mail';
import MailDebug from './mail/providers/Debug';
import MailFile from './mail/providers/File';


const convert = require('koa-convert');

spice.classes = {};
spice.classes.Event = Event;
spice.classes.Model = Model;
spice.classes.Crypt = Crypt;
spice.classes.Serializer = Serializer;
spice.classes.RestHelper = RestHelper;
spice.classes.validate = validate;
spice.classes.Mail = Mail;
spice.classes.mail = {providers:{}};
spice.classes.mail.providers.Debug = MailDebug;
spice.classes.mail.providers.File = MailFile;
export let status = Status;
export class Spice {
    constructor() {

    }

    async init(app){
        try{
            app.use(cors());
            require('koa-qs')(app, 'extended');
            app.use(koaBody({formLimit:"1gb", multipart: true, formidable:{uploadDir: './storage/uploads'},}));
            app.use(convert(validate()));
            spice.app = app;
            spice.config = await require('./loaders/config')();
            spice.io = io;
            require('./loaders');
            io.attach(app);
            return spice;
        }catch(e){
            console.log(e.stack);
        }
    }

    async init_spa(vue){
        try{
            window.spice = {};
            //window.config = await require('./loaders/config')();
        }catch(e){
            console.log(e.stack)
        }
    }
}

export default Spice;